@@ -1,6 +1,6 @@
 /* empty - run processes under pseudo-terminal sessions
  * 
- * Copyright (C) 2005-2007 Mikhail E. Zakharov
+ * Copyright (C) 2005-2009 Mikhail E. Zakharov
  * empty was written by Mikhail E. Zakharov. This software was based on the 
  * basic idea of pty version 4.0 Copyright (c) 1992, Daniel J. Bernstein, but
  * no code was ported from pty4.
@@ -105,7 +105,7 @@
 
 #define tmpdir "/tmp"
 #define program "empty"
-#define version "0.6.15b" 
+#define version "0.6.17b" 
 
 /* -------------------------------------------------------------------------- */
 static void usage(void);
@@ -173,6 +173,10 @@ int main (int argc, char *argv[]) {
 
 	time_t	stime, ntime;
 	struct	timeval tv;
+
+	int	fl_state = 2;		/* 0 - in	>>>
+					   1 - out	<<<
+					   2 - unknown */
 	
 /* semaphores */
 #ifdef _POSIX_SEMAPHORES
@@ -366,9 +370,12 @@ int main (int argc, char *argv[]) {
 			FD_SET(ifd, &rfd);
 			n = select(ifd + 1, &rfd, 0, 0, &tv);
 			if (n < 0 && errno != EINTR)
+#ifdef __linux__
+				tv.tv_sec = timeout; //fix because struct was set to 0; thanks "David Hofstee" <davidh@blinker.nl>
+#endif
 				perrx(255, "Fatal select()");
 
-			if (FD_ISSET(ifd, &rfd)) {
+			if (n > 0 && FD_ISSET(ifd, &rfd)) {
 				if ((cc = read(ifd, buf, bs)) == -1)
 					(void)perrx(255, "Fatal read from IN FIFO");
 				if (write(1, buf, cc) == -1)
@@ -560,7 +567,16 @@ int main (int argc, char *argv[]) {
 	  	  (void)perrxslog(255, "Fatal tcsetpgrp()");
 
 #endif
-		tt.c_lflag &= ~ECHO;
+
+#if defined(__SVR4)
+		/* Setup terminal parameters for Solaris to work under cron or java.
+		Thanks to "lang qiu" <qiulang@gmail.com> for the initial line of parameters */
+		tt.c_lflag = ISIG | ICANON | ECHOE | ECHOK | ECHOCTL | ECHOKE | IEXTEN; 
+		tt.c_oflag = TABDLY | OPOST;
+		tt.c_iflag = BRKINT | IGNPAR | ISTRIP | ICRNL | IXON | IMAXBEL;
+		tt.c_cflag = CBAUD | CS8 | CREAD; 
+#endif
+		tt.c_lflag &= ~ECHO; 
 		(void)tcsetattr(STDIN_FILENO, TCSAFLUSH, &tt);
 
 		strncpy(buf, argv[0], sizeof(buf) - 1);
@@ -593,16 +609,28 @@ int main (int argc, char *argv[]) {
 		if (n > 0 || errno == EINTR) {
 			if (FD_ISSET(ifd, &rfd)) 
 				if ((cc = read(ifd, buf, sizeof(buf))) > 0) {
+					/* our input */
 					(void)write(master, buf, cc);
-					if (lfd)
+					if (lfd) {
+						if (fl_state != 1) {
+							(void)write(lfd, ">>>", 3);
+							fl_state = 1;
+						}
 						(void)write(lfd, buf, cc);
+					}
 				}
 
 			if (FD_ISSET(master, &rfd))
 				if ((cc = read(master, buf, sizeof(buf))) > 0) {
+					/* remote output */
 					(void)write(ofd, buf, cc);
-					if (lfd)
+					if (lfd) {
+						if (fl_state != 0) {
+							(void)write(lfd, "<<<", 3);
+							fl_state = 0;
+						}
 						(void)write(lfd, buf, cc);
+					}
 				}
 		}
 	}
@@ -858,10 +886,13 @@ int watch4str(int ifd, int ofd, int argc, char *argv[],
 	for (;;) {
 		FD_SET(ifd, &rfd);
 		n = select(ifd + 1, &rfd, 0, 0, &tv);
+#ifdef __linux__
+		tv.tv_sec = timeout; //fix because struct was set to 0; thanks David Hofstee" <davidh@blinker.nl>
+#endif
 		if (n < 0 && errno != EINTR) 
 			perrx(255, "Fatal select()");
 
-		if (FD_ISSET(ifd, &rfd)) {
+		if (n > 0 && FD_ISSET(ifd, &rfd)) {
 			largv = 0;
 			if ((cc = read(ifd, buf + largv, sizeof(buf) - largv)) > 0) {
 				stime = time(0);
